Observe
Enums
When included in a GET request, the Observe Option extends the GET
method so it does not only retrieve a current representation of the
target resource, but also requests the server to add or remove an
entry in the list of observers of the resource depending on the
option value. The list entry consists of the client endpoint and the
token specified by the client in the request. Possible values are: