pub struct Block(_);Expand description
Three items of information may need to be transferred in a Block (Block1 or Block2) option:
- the size of the block (
Block::size) - whether more blocks are following (
Block::more) - the relative number of the block (
Block::num) within a sequence of blocks with the given size.
Implementations